import { compile as platformCompile } from '@lwc/sfdc-lwc-compiler';
import { Workspace } from '../../types/index.js';
import { compile } from './lexBundler';
// Mock dependencies
vi.mock('@lwc/sfdc-lwc-compiler', () => ({
    compile: vi.fn(),
}));
vi.mock('../util/telemetry.js', () => ({
    measureOperationTime: vi.fn((name, fn) => fn()),
}));
const mockCompileOptions = {
    enableDynamicComponents: true,
    enableLightningWebSecurityTransforms: true,
    experimentalComplexExpressions: true,
    apiVersion: 50,
    name: 'testModule',
    namespace: 'c',
    outputConfig: { sourcemap: true },
};
describe('compile', () => {
    it('should compile successfully', async () => {
        const ctx = {
            config: {
                workspace: Workspace.Internal,
            },
            logger: {
                error: vi.fn(),
            },
            moduleGraph: {},
        };
        const module = {
            getSources: vi.fn().mockResolvedValue({ 'test.js': 'console.log("test");' }),
        };
        const part = 'testPart';
        vi.mocked(platformCompile).mockResolvedValue({
            success: true,
            results: [{ code: 'compiled code' }],
            diagnostics: [],
            version: '1.0.0',
            timings: {},
        });
        const result = await compile(ctx, { module, part }, mockCompileOptions);
        expect(platformCompile).toHaveBeenCalledWith({
            bundle: {
                files: {
                    'test.js': 'console.log("test");',
                },
                name: 'testModule',
                namespace: 'c',
                options: {
                    allowDynamicComponents: true,
                    apiVersion: 50,
                    disableProcessNodeEnv: true,
                    dynamicImport: 'allowed',
                    enableHmr: true,
                    enableLightningWebSecurityTransforms: true,
                    enableTemplateExpressions: true,
                    forceNamedExports: true,
                    strictCSSTokenCollection: true,
                },
                type: 'internal',
            },
            compilerConfigs: [
                {
                    outputConfig: {
                        minify: false,
                        sourcemap: true,
                    },
                },
            ],
        });
        expect(result).toEqual({
            code: 'compiled code',
            warnings: [],
        });
        expect(ctx.logger.error).not.toHaveBeenCalledWith(expect.any(String));
    });
    it('should handle compile errors', async () => {
        const ctx = {
            config: {
                workspace: Workspace.Internal,
            },
            logger: {
                error: vi.fn(),
            },
            moduleGraph: {},
        };
        const module = {
            getSources: vi.fn().mockResolvedValue({ 'test.js': 'console.log("test");' }),
        };
        const part = 'testPart';
        vi.mocked(platformCompile).mockResolvedValue({
            version: '1.0.0',
            timings: {},
            success: false,
            results: [],
            diagnostics: [{ code: 1010, level: 1, message: 'Error message' }],
        });
        const result = await compile(ctx, { module, part }, mockCompileOptions);
        expect(result).toEqual({
            code: undefined,
            warnings: [
                {
                    code: 1010,
                    level: 1,
                    message: 'Error message',
                },
            ],
        });
        expect(ctx.logger.error).toHaveBeenCalledWith(`Failed to compile ${part}. \n\tDiagnostics: [{"code":1010,"level":1,"message":"Error message"}]`);
    });
});
//# sourceMappingURL=lexBundler.spec.js.map