import { compile } from '../bundler/lexBundler.js';
import { decodeModulePath } from '../util/module-util.js';
/**
 * Class representing a client that has connected to the dev server.
 * This class represents a client that is connected and ready to receive hot module updates.
 * After the hmr-client is connected, all communication between itself and lwc-dev-server are handled
 * by this class.
 */
export class Client {
    constructor(clientId, url, connection, ctx) {
        this.clientId = clientId;
        this.url = url;
        this.connection = connection;
        this.connection.receive(this.messageCallback.bind(this));
        this.ctx = ctx;
        this.compileOptions = new Map();
    }
    onClose(callback) {
        this.connection.onClose(callback);
    }
    send(data) {
        this.connection.send(data);
    }
    async messageCallback(data) {
        const { ctx: { moduleGraph }, } = this;
        switch (data.type) {
            // In coming hot module
            case 'fetch': {
                const { data: { modulePath }, } = data;
                const modulePart = this.ctx.moduleGraph.getModulePart(modulePath);
                if (!modulePart) {
                    const error = {
                        type: 'error',
                        data: {
                            message: `Fetch of ${modulePath} failed as it does not exist in dev server workspace`,
                        },
                    };
                    this.connection.send(error);
                    break;
                }
                this.sendUpdatedModule(modulePart);
                break;
            }
            case 'register': {
                const moduleParts = [];
                for (const { modulePath, hash, compileOptions } of data.data.modules) {
                    const { id, part } = decodeModulePath(modulePath);
                    moduleParts.push({ id, part, hash });
                    this.compileOptions.set(id, compileOptions);
                }
                // Inform the hmr-client of available updates, this ensures that any local changes
                // that haven't been deployed to the remote app are updated upon client registration.
                const staleModules = moduleGraph.getStaleModules(moduleParts);
                if (staleModules.size > 0) {
                    this.send({
                        type: 'update',
                        data: Array.from(staleModules).map((modulePath) => ({ modulePath })),
                    });
                }
                break;
            }
            case 'invalidate': {
                const { data: { modulePath }, } = data;
                const modulePart = this.ctx.moduleGraph.getModulePart(modulePath);
                if (!modulePart) {
                    const error = {
                        type: 'error',
                        data: {
                            message: `Fetch of ${modulePath} failed as it does not exist in dev server workspace`,
                        },
                    };
                    this.connection.send(error);
                    break;
                }
                // when we invalidate a module, we send the updated entry module
                const entryPart = modulePart.module.getEntryPart();
                this.sendUpdatedModule({ ...modulePart, part: entryPart });
                break;
            }
        }
    }
    async sendUpdatedModule(modulePart) {
        const modulePath = modulePart.module.id(modulePart.part);
        this.ctx.logger.debug(`[LWC Dev Server] Compiling ${modulePath}`);
        const compileOptions = this.compileOptions.get(modulePart.module.id());
        if (!compileOptions) {
            this.ctx.logger.error(`Found no compile option for module path: ${modulePart.module.id()}, check the __lwc_hmr_hot.register() call.`);
            throw new Error(`No Compile Options for module path: ${modulePart.module.id()}`);
        }
        const result = await compile(this.ctx, modulePart, compileOptions);
        if (!result.code) {
            this.ctx.logger.error(`Compiling ${modulePath} failed and there are compiler warnings: ${JSON.stringify(result.warnings)}`);
            return;
        }
        this.ctx.logger.debug(`[LWC Dev Server] Sending an updated ${modulePath}`);
        this.connection.send({
            type: 'module-update',
            data: [{ modulePath, src: result.code }],
        });
        this.ctx.config.lifecycle?.emitTelemetry({
            eventName: 'compile-success',
            module: modulePath,
        });
    }
    close() {
        this.connection.close();
    }
}
//# sourceMappingURL=client.js.map