import { compile } from '../bundler/lexBundler.js';
import { Module } from '../moduleGraph/module.js';
import { Client } from './client.js';
// Mock dependencies
vi.mock('../bundler/lexBundler.js', () => ({
    compile: vi.fn(),
}));
describe('Client', () => {
    let clientId;
    let url;
    let connection;
    let ctx;
    let client;
    beforeEach(() => {
        clientId = 'test-client-id';
        url = 'http://localhost';
        connection = {
            receive: vi.fn(),
            send: vi.fn(),
            onClose: vi.fn(),
            close: vi.fn(),
        };
        ctx = {
            logger: {
                debug: vi.fn(),
            },
            moduleGraph: {
                getModulePart: vi.fn(),
                getStaleModules: vi.fn(),
            },
            config: {
                lifecycle: {
                    emitTelemetry: vi.fn(),
                },
            },
        };
        client = new Client(clientId, url, connection, ctx);
    });
    it('should initialize with correct properties', () => {
        expect(client.clientId).toBe(clientId);
        expect(client.url).toBe(url);
        expect(client.connection).toBe(connection);
        expect(client.ctx).toBe(ctx);
        expect(connection.receive).toHaveBeenCalledWith(expect.any(Function));
    });
    it('should handle onClose callback', () => {
        const callback = vi.fn();
        client.onClose(callback);
        expect(connection.onClose).toHaveBeenCalledWith(callback);
    });
    it('should send data', () => {
        const data = { type: 'update', data: [] };
        client.send(data);
        expect(connection.send).toHaveBeenCalledWith(data);
    });
    it('should handle fetch message callbacks', async () => {
        vi.mocked(ctx.moduleGraph.getStaleModules).mockReturnValue(new Set());
        await client.messageCallback({
            type: 'register',
            data: {
                modules: [
                    {
                        compileOptions: { name: 'name', namespace: 'ns' },
                        hash: 'hash1',
                        modulePath: 'ns/name/cmp.js',
                    },
                ],
            },
        });
        const data = {
            type: 'fetch',
            data: { modulePath: 'ns/name/cmp.js' },
        };
        const modulePart = { module: new Module('', 'ns', 'name', {}), part: 'cmp.js' };
        vi.mocked(ctx.moduleGraph.getModulePart).mockReturnValue(modulePart);
        vi.mocked(compile).mockResolvedValue({ code: 'compiled code', warnings: [] });
        await client.messageCallback(data);
        expect(ctx.logger.debug).toHaveBeenCalledWith(`[LWC Dev Server] Compiling ns/name/cmp.js`);
        expect(compile).toHaveBeenCalledWith(ctx, modulePart, { name: 'name', namespace: 'ns' });
        expect(ctx.logger.debug).toHaveBeenCalledWith(`[LWC Dev Server] Sending an updated ns/name/cmp.js`);
        expect(connection.send).toHaveBeenCalledWith({
            type: 'module-update',
            data: [{ modulePath: 'ns/name/cmp.js', src: 'compiled code' }],
        });
        expect(ctx.config.lifecycle.emitTelemetry).toHaveBeenCalledWith({
            eventName: 'compile-success',
            module: 'ns/name/cmp.js',
        });
    });
    it('should handle fetch message callback with error', async () => {
        const data = {
            type: 'fetch',
            data: { modulePath: 'src/ns/name/cmp.js' },
        };
        vi.mocked(ctx.moduleGraph.getModulePart).mockReturnValue(undefined);
        await client.messageCallback(data);
        expect(connection.send).toHaveBeenCalledWith({
            type: 'error',
            data: {
                message: `Fetch of src/ns/name/cmp.js failed as it does not exist in dev server workspace`,
            },
        });
    });
    it('should handle register message callback', async () => {
        const data = {
            type: 'register',
            data: {
                modules: [
                    {
                        modulePath: 'ns/name/cmp.js',
                        hash: 'hash1',
                        compileOptions: { name: 'name', namespace: 'ns' },
                    },
                ],
            },
        };
        const staleModules = new Set(['ns/name/cmp.js']);
        vi.mocked(ctx.moduleGraph.getStaleModules).mockReturnValue(staleModules);
        await client.messageCallback(data);
        expect(ctx.moduleGraph.getStaleModules).toHaveBeenCalledWith([
            {
                hash: 'hash1',
                id: 'ns/name',
                part: 'cmp.js',
            },
        ]);
        expect(connection.send).toHaveBeenCalledWith({
            type: 'update',
            data: Array.from(staleModules).map((modulePath) => ({ modulePath })),
        });
    });
    it('should close the connection', () => {
        client.close();
        expect(connection.close).toHaveBeenCalled();
    });
});
//# sourceMappingURL=client.spec.js.map