import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
import { UNRECOGNIZED_MESSAGE_TYPE } from '../../labels.js';
import { Connection } from './connection.js';
describe('Connection', () => {
    let mockSocket;
    let connection;
    const mockLogger = {
        error: vi.fn(),
    };
    beforeEach(() => {
        mockSocket = {
            close: vi.fn(),
            on: vi.fn(),
            send: vi.fn(),
            addEventListener: vi.fn(),
        };
        connection = new Connection(mockSocket, mockLogger);
    });
    afterEach(() => {
        vi.clearAllMocks();
    });
    it('should close the WebSocket connection', () => {
        connection.close();
        expect(mockSocket.close).toHaveBeenCalled();
    });
    it('should register a close event listener', () => {
        const closeCallback = vi.fn();
        connection.onClose(closeCallback);
        expect(mockSocket.on).toHaveBeenCalledWith('close', closeCallback);
    });
    it('should send data over the WebSocket connection', () => {
        const mockData = { type: 'update', data: [{ modulePath: '' }] };
        connection.send(mockData);
        expect(mockSocket.send).toHaveBeenCalledWith(JSON.stringify(mockData));
    });
    it('should receive data from the WebSocket connection', () => {
        const mockMessageEvent = {
            data: JSON.stringify({ type: 'register', data: {} }),
        };
        const receiveCallback = vi.fn();
        connection.receive(receiveCallback);
        const addEventListenerCall = vi.mocked(mockSocket).addEventListener.mock.calls[0];
        const eventListener = addEventListenerCall[1];
        eventListener(mockMessageEvent);
        expect(receiveCallback).toHaveBeenCalledWith({ type: 'register', data: {} });
    });
    it('should log an error for unrecognized message types', () => {
        const mockMessageEvent = {
            data: new ArrayBuffer(8), // Non-string data to simulate unrecognized message type
        };
        connection.receive(vi.fn());
        const addEventListenerCall = vi.mocked(mockSocket).addEventListener.mock.calls[0];
        const eventListener = addEventListenerCall[1];
        eventListener(mockMessageEvent);
        expect(mockLogger.error).toHaveBeenCalledWith(UNRECOGNIZED_MESSAGE_TYPE);
    });
});
//# sourceMappingURL=connection.spec.js.map