/*
 * Copyright (c) 2018, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: MIT
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/MIT
 */
import * as http from 'http';
import * as https from 'https';
import { URL } from 'node:url';
import { WebSocketServer } from 'ws';
import { templateString } from '../../utils/utils.js';
import { WSS_STARTUP_FAILURE } from '../../labels.js';
import { decrypt } from '../util/aes.js';
import { Connection } from './connection.js';
/**
 * Special mechanism to take internal users through an authentication flow.
 */
function requestHandler(req, res) {
    // the second arg could really be anything, we're just interested in searchParams
    // also req.url can never be undefined, it's just bad typing
    const url = new URL(req.url, 'http://localhost');
    if (req.method !== 'GET' || url.pathname !== '/') {
        res.writeHead(404, { 'Content-Type': 'text/html' });
        return res.end(`404 Not Found`);
    }
    const redirectUrl = url.searchParams.get('redirectUrl');
    if (redirectUrl) {
        res.statusCode = 302;
        res.setHeader('Location', redirectUrl);
        return res.end();
    }
    res.writeHead(200, { 'Content-Type': 'text/html' });
    res.end(`Visit https://&lt;salesforce-url&gt;?0.aura.mode=devpreview`);
}
const TRUSTED_ORIGINS = [
    /\.sfdcdev\.force\.com:[0-9]+$/,
    /\.w[a-z]\.crm\.dev:6101$/,
    /\.lightning\.force\.com$/,
    /\.lightning\.pc-rnd\.force\.com$/,
];
function isFromTrustedOrigin(request) {
    const { headers } = request;
    const { origin } = headers;
    return (
    // Only upgrade requests that are coming in from the hmr-client
    // https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications#protocols
    headers['sec-websocket-protocol'] === 'lightning-dev-preview' &&
        origin &&
        TRUSTED_ORIGINS.some((trustedOrigin) => trustedOrigin.test(origin)));
}
/**
 * Wrapper over WebSocketServer that does authentication and produces `Connection`s
 */
export class AuthenticatedWebSocketServer {
    constructor({ port, https: httpsConfig, identityToken }, logger) {
        this.logger = logger;
        try {
            // Start a WebSocketServer disconnected from the http server, allows us to handle authantication before upgrading ws connections
            this.wss = new WebSocketServer({ noServer: true });
            this.identityToken = identityToken;
            const httpServer = http.createServer();
            httpServer.maxRequestsPerSocket = 1;
            // Special handler for the internal usecase
            httpServer.on('request', requestHandler);
            const upgradeHandler = (request, socket, head) => {
                if (!isFromTrustedOrigin(request)) {
                    socket.write('HTTP/1.1 401 Unauthorized\r\n\r\n');
                    socket.destroy();
                    return;
                }
                this.wss.handleUpgrade(request, socket, head, (ws) => {
                    this.wss.emit('connection', ws, request);
                });
            };
            // Handle upgrade of connections after verifying the validity
            httpServer.on('upgrade', upgradeHandler);
            httpServer.listen(port, () => {
                logger.info(`[LWC Dev Server] LWC Dev Server started at port:${port}`);
            });
            if (httpsConfig) {
                const httpsServer = https.createServer(httpsConfig);
                httpsServer.on('request', requestHandler);
                httpsServer.on('upgrade', upgradeHandler);
                httpsServer.listen(httpsConfig.port, () => {
                    logger.info(`[LWC Dev Server] LWC HTTPS Dev Server started at port:${httpsConfig.port}`);
                });
                this.wss.on('close', () => {
                    httpsServer.close();
                });
            }
            // When the WebSocketServer is being closed, close the http server as well.
            this.wss.on('close', () => {
                httpServer.close();
            });
        }
        catch (e) {
            const errorMsg = templateString(WSS_STARTUP_FAILURE, [e]);
            logger.error(errorMsg);
            throw e;
        }
    }
    close() {
        this.wss.close();
    }
    onNewConnection(callback) {
        this.wss.on('connection', (ws, request) => {
            const connection = new Connection(ws, this.logger);
            const searchParams = new URL(request.url, 'http://localhost').searchParams;
            if (this.identityToken) {
                const sessionTokenCipher = searchParams.get('sessionTokenCipher');
                if (!sessionTokenCipher) {
                    this.logger.error('sessionTokenCipher not present. Terminating.');
                    ws.close();
                    return;
                }
                const sessionToken = decrypt(sessionTokenCipher, this.identityToken);
                connection.send({
                    type: 'auth-response',
                    data: { sessionToken: sessionToken },
                });
            }
            const referer = searchParams.get('referer');
            if (!referer) {
                this.logger.error('Referer param not present. Terminating.');
                ws.close();
                return;
            }
            callback(connection, referer);
        });
    }
}
//# sourceMappingURL=web-socket-server.js.map