import * as http from 'http';
import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
import { WebSocketServer } from 'ws';
import { Workspace } from '../../types';
import * as aes from '../util/aes';
import { AuthenticatedWebSocketServer } from './web-socket-server.js';
import { Connection } from './connection';
// Mock dependencies
vi.mock('ws');
vi.mock('http');
vi.mock('https');
vi.mock('../util/aes');
vi.mock('./connection');
describe('AuthenticatedWebSocketServer', () => {
    const logger = { error: vi.fn() };
    let serverConfig;
    let wsServerMock;
    let httpServerMock;
    beforeEach(() => {
        serverConfig = {
            port: 3000,
            identityToken: 'mock-identity-token',
            rootDir: '',
            paths: [],
            workspace: Workspace.Internal,
        };
        wsServerMock = {
            on: vi.fn(),
            handleUpgrade: vi.fn(),
            emit: vi.fn(),
            close: vi.fn(),
        };
        httpServerMock = {
            on: vi.fn(),
            listen: vi.fn(),
            close: vi.fn(),
        };
        vi.mocked(WebSocketServer).mockImplementation(() => wsServerMock);
        vi.mocked(http.createServer).mockReturnValue(httpServerMock);
    });
    afterEach(() => {
        vi.restoreAllMocks();
    });
    it('should create an instance of AuthenticatedWebSocketServer and start the HTTP server', () => {
        new AuthenticatedWebSocketServer(serverConfig, logger);
        expect(http.createServer).toHaveBeenCalled();
        expect(httpServerMock.on).toHaveBeenCalledWith('request', expect.any(Function));
        expect(httpServerMock.on).toHaveBeenCalledWith('upgrade', expect.any(Function));
        expect(httpServerMock.listen).toHaveBeenCalledWith(serverConfig.port, expect.any(Function));
    });
    it('should handle WebSocket upgrade requests from trusted origins', () => {
        const socketMock = {
            write: vi.fn(),
            destroy: vi.fn(),
        };
        new AuthenticatedWebSocketServer(serverConfig, logger);
        const upgradeHandler = vi
            .mocked(httpServerMock)
            .on.mock.calls.find((call) => call[0] === 'upgrade')[1];
        const requestMock = {
            headers: {
                'sec-websocket-protocol': 'lightning-dev-preview',
                origin: 'https://mock-origin.lightning.force.com',
            },
            url: '/',
        };
        const headMock = Buffer.alloc(0);
        // @ts-expect-error can't mock the entire socket
        upgradeHandler(requestMock, socketMock, headMock);
        expect(wsServerMock.handleUpgrade).toHaveBeenCalledWith(requestMock, socketMock, headMock, expect.any(Function));
    });
    it('should reject WebSocket upgrade requests from untrusted origins', () => {
        const httpServerMock = {
            on: vi.fn(),
            listen: vi.fn(),
            close: vi.fn(),
        };
        vi.mocked(http.createServer).mockReturnValue(httpServerMock);
        const socketMock = {
            write: vi.fn(),
            destroy: vi.fn(),
        };
        new AuthenticatedWebSocketServer(serverConfig, logger);
        const upgradeHandler = vi
            .mocked(httpServerMock)
            .on.mock.calls.find((call) => call[0] === 'upgrade')[1];
        const requestMock = {
            headers: {
                'sec-websocket-protocol': 'unknown-protocol',
                origin: 'https://untrusted-origin.com',
            },
            url: '/',
        };
        const headMock = Buffer.alloc(0);
        // @ts-expect-error can't mock the entire socket
        upgradeHandler(requestMock, socketMock, headMock);
        expect(socketMock.write).toHaveBeenCalledWith('HTTP/1.1 401 Unauthorized\r\n\r\n');
        expect(socketMock.destroy).toHaveBeenCalled();
    });
    it('should close the WebSocketServer when close() is called', () => {
        const authenticatedWSS = new AuthenticatedWebSocketServer(serverConfig, logger);
        authenticatedWSS.close();
        expect(wsServerMock.close).toHaveBeenCalled();
    });
    it('should handle a new WebSocket connection and send an auth-response if identityToken is provided', () => {
        const connectionMock = {
            send: vi.fn(),
        };
        vi.mocked(Connection).mockReturnValue(connectionMock);
        const wsMock = {};
        const requestMock = {
            url: '/?sessionTokenCipher=mock-cipher&referer=https://mock-referer.com',
        };
        const decryptSpy = vi.spyOn(aes, 'decrypt').mockReturnValue('decrypted-token');
        const authenticatedWSS = new AuthenticatedWebSocketServer(serverConfig, logger);
        const callback = vi.fn();
        authenticatedWSS.onNewConnection(callback);
        vi.mocked(wsServerMock)
            .on.mock.calls.find((call) => call[0] === 'connection')[1]
            .call(wsServerMock, wsMock, requestMock);
        expect(Connection).toHaveBeenCalledWith(wsMock, logger);
        expect(decryptSpy).toHaveBeenCalledWith('mock-cipher', serverConfig.identityToken);
        expect(connectionMock.send).toHaveBeenCalledWith({
            type: 'auth-response',
            data: { sessionToken: 'decrypted-token' },
        });
        expect(callback).toHaveBeenCalledWith(connectionMock, 'https://mock-referer.com');
    });
    it('should close WebSocket connection if sessionTokenCipher is missing', () => {
        const connectionMock = {
            send: vi.fn(),
        };
        vi.mocked(Connection).mockReturnValue(connectionMock);
        const wsMock = {
            close: vi.fn(),
        };
        const requestMock = {
            url: '/',
            headers: {
                referer: 'https://mock-referer.com',
            },
        };
        const authenticatedWSS = new AuthenticatedWebSocketServer(serverConfig, logger);
        const callback = vi.fn();
        authenticatedWSS.onNewConnection(callback);
        vi.mocked(wsServerMock)
            .on.mock.calls.find((call) => call[0] === 'connection')[1]
            .call(wsServerMock, wsMock, requestMock);
        expect(wsMock.close).toHaveBeenCalled();
        expect(callback).not.toHaveBeenCalled();
    });
    it('should close WebSocket connection if referer header is missing', () => {
        const wsMock = {
            close: vi.fn(),
        };
        const requestMock = {
            url: '/?sessionTokenCipher=mock-cipher',
            headers: {},
        };
        const authenticatedWSS = new AuthenticatedWebSocketServer(serverConfig, logger);
        const callback = vi.fn();
        authenticatedWSS.onNewConnection(callback);
        vi.mocked(wsServerMock)
            .on.mock.calls.find((call) => call[0] === 'connection')[1]
            .call(wsServerMock, wsMock, requestMock);
        expect(wsMock.close).toHaveBeenCalled();
        expect(callback).not.toHaveBeenCalled();
    });
});
//# sourceMappingURL=web-socket-server.spec.js.map