import { LWCServer } from './index.js';
import { AuthenticatedWebSocketServer } from './connection/web-socket-server.js';
import { ModuleGraph } from './moduleGraph/index.js';
import { Client } from './connection/client.js';
import { Connection } from './connection/connection.js';
vi.mock('./connection/web-socket-server.js', () => ({
    AuthenticatedWebSocketServer: vi.fn(() => ({
        onNewConnection: vi.fn(),
        close: vi.fn(),
    })),
}));
vi.mock('./moduleGraph/index.js', () => {
    const ModuleGraph = vi.fn();
    ModuleGraph.prototype.on = vi.fn();
    ModuleGraph.prototype.close = vi.fn();
    return { ModuleGraph };
});
vi.mock('./consoleLogger.js');
vi.mock('./connection/client.js', () => {
    const Client = vi.fn(() => ({
        send: vi.fn(),
        onClose: vi.fn(),
    }));
    return { Client };
});
vi.mock('./connection/connection.js');
describe('LWCServer', () => {
    let config;
    const logger = {
        info: vi.fn(),
        error: vi.fn(),
    };
    let wss;
    let lwcServer;
    beforeEach(() => {
        config = {
            paths: ['/some/path'],
            identityToken: 'test-token',
            maxClientCount: 2,
        };
        wss = new AuthenticatedWebSocketServer(config, logger);
        lwcServer = new LWCServer(config, wss, logger);
    });
    afterEach(() => {
        vi.clearAllMocks();
    });
    it('should initialize the server with correct components', () => {
        expect(ModuleGraph).toHaveBeenCalledWith(config, logger);
    });
    it('should handle new connections and enforce max client limit', () => {
        const mockUrl = 'ws://localhost';
        const onNewConnection = vi.mocked(wss.onNewConnection).mock.calls[0][0];
        // @ts-expect-error: Values are mocked
        const connection = new Connection();
        onNewConnection(connection, mockUrl);
        expect(Client).toHaveBeenCalledTimes(1);
        expect(lwcServer.clients.size).toBe(1);
        onNewConnection(connection, mockUrl);
        onNewConnection(connection, mockUrl); // This should exceed max client count
        expect(Client).toHaveBeenCalledTimes(2); // Only two clients allowed
        expect(connection.close).toHaveBeenCalledTimes(1); // Third connection should be closed
        expect(lwcServer.clients.size).toBe(2);
    });
    it('should close and stop server when moduleGraph closes', () => {
        const onError = vi
            .mocked(vi.mocked(ModuleGraph).mock.instances[0].on)
            .mock.calls.find(([eventName]) => eventName === 'close')[1];
        // @ts-expect-error: can't fix :(
        onError();
        expect(wss.close).toHaveBeenCalled();
    });
    it('should broadcast to all clients when moduleGraph update is emitted', () => {
        const onNewConnection = vi.mocked(wss.onNewConnection).mock.calls[0][0];
        // @ts-expect-error: Values are mocked
        onNewConnection(new Connection(), 'ws://localhost');
        // @ts-expect-error: Values are mocked
        onNewConnection(new Connection(), 'ws://localhost');
        const firstClient = vi.mocked(Client).mock.results[0].value;
        const secondClient = vi.mocked(Client).mock.results[1].value;
        const onUpdate = vi
            .mocked(vi.mocked(ModuleGraph).mock.instances[0].on)
            .mock.calls.find(([eventName]) => eventName === 'update')[1];
        // @ts-expect-error: can't fix :(
        onUpdate();
        expect(firstClient.send).toHaveBeenCalled();
        expect(secondClient.send).toHaveBeenCalled();
        // close second client and clear its mocks
        vi.mocked(secondClient.onClose).mock.calls[0][0]();
        vi.mocked(secondClient.send).mockClear();
        // @ts-expect-error: can't fix :(
        onUpdate();
        expect(firstClient.send).toHaveBeenCalled();
        expect(secondClient.send).not.toHaveBeenCalled();
    });
    it('should stop the server and release resources', () => {
        lwcServer.stopServer();
        expect(logger.info).toHaveBeenCalledWith('[LWC Dev Server] Stopping the dev server');
        expect(vi.mocked(ModuleGraph).mock.instances[0].close).toHaveBeenCalled();
        expect(lwcServer.wss.close).toHaveBeenCalled();
        expect(logger.info).toHaveBeenCalledWith('[LWC Dev Server] Succesfully stopped dev server');
    });
});
//# sourceMappingURL=index.spec.js.map