import { EventEmitter } from 'events';
import path from 'path';
import { startWatch } from '../watcher/index.js';
import { decodeModulePath } from '../util/module-util.js';
import { getWorkspaceScanner } from './workspaceScanner/index.js';
import { InvalidModuleError } from './module.js';
export class ModuleGraph extends EventEmitter {
    constructor(config, logger) {
        super();
        this.modules = new Map();
        this.watcher = startWatch(config.paths);
        this.logger = logger;
        this.workspaceScanner = getWorkspaceScanner(config, logger);
        // init() may be called multiple times in the future(when a directory is deleted or addeds)
        this.init();
    }
    init() {
        const modules = this.workspaceScanner.getModules();
        for (const module of modules) {
            this.modules.set(module.id(), module);
        }
        if (modules.length) {
            this.logger.debug(`[Module Graph] Found ${modules.length} LWC components in the current workspace.`);
        }
        else {
            this.logger.debug(`[Module Graph] Found no LWC components in the current workspace.`);
        }
        this.watcher.on('change', this.fileChangeHandler.bind(this));
        this.watcher.on('add', this.fileAddHandler.bind(this));
        this.watcher.on('unlink', this.fileDeleteHandler.bind(this));
        this.watcher.on('error', (event) => {
            this.logger.error('[LWC Dev Server] Encountered error while watching files');
            this.logger.debug(`[LWC Dev Server] File watcher error: ${event.message}`);
            this.emit('close');
        });
    }
    /**
     * Gets the stale modules from the list of modules
     * @param modules
     * @returns Modules paths that are determined to be stale wrt the content monitored by the dev server
     */
    getStaleModules(modules) {
        const staleModules = [];
        const unTrackedModules = [];
        modules.forEach(({ id, part, hash }) => {
            const module = this.modules.get(id);
            if (!module) {
                unTrackedModules.push(id);
                return;
            }
            if (!module.getPart(part) || module.getPart(part).moduleSourceHash !== hash) {
                staleModules.push(module);
            }
        });
        if (unTrackedModules.length) {
            this.logger.trace(`[Module Graph] Following modules are unknown to dev server: \n\t ${unTrackedModules.join(',\n\t')}`);
        }
        if (staleModules.length) {
            this.logger.debug(`[Module Graph] Following modules have a different version: \n\t ${staleModules.map((module) => module.id()).join(',\n\t')}`);
        }
        const staleModuleBoundaries = new Set();
        for (const module of staleModules) {
            staleModuleBoundaries.add(module.getEntryFile());
        }
        return staleModuleBoundaries;
    }
    fileChangeHandler(filePath) {
        const modulePart = this.getModulePartByFile(filePath);
        if (!modulePart)
            return;
        const { module, part } = modulePart;
        if (module.addOrUpdatePart(part)) {
            this.logger.info(`[LWC Dev Server] File '${part}' modified - updating.`);
            this.emit('update', module.getModuleBoundary(part));
        }
    }
    fileAddHandler(filePath) {
        const modulePart = this.getModulePartByFile(filePath);
        if (!modulePart)
            return;
        const { module, part } = modulePart;
        if (module.addOrUpdatePart(part)) {
            this.logger.info(`[LWC Dev Server] File '${part}' added - updating.`);
            this.emit('update', module.getEntryFile());
        }
    }
    fileDeleteHandler(filePath) {
        const modulePart = this.getModulePartByFile(filePath);
        if (!modulePart)
            return;
        const { module, part } = modulePart;
        module.deletePart(part);
        try {
            // When files are deleted, we re-render the entire component, so we update the entry file itself
            this.logger.info(`[LWC Dev Server] File '${filePath}' deleted - updating.`);
            const entryFile = module.getEntryFile();
            this.emit('update', entryFile);
        }
        catch (error) {
            if (error instanceof InvalidModuleError) {
                this.logger.info(`[LWC Dev Server] Entry File '${filePath}' deleted. Deleted module - updating.`);
                this.modules.delete(module.id());
                this.emit('delete-module', module.id());
            }
        }
    }
    getModulePart(modulePath) {
        const { id, part } = decodeModulePath(modulePath);
        const module = this.modules.get(id);
        if (module)
            return { module, part };
    }
    getModulePartByFile(file) {
        let module;
        for (const m of this.modules.values()) {
            if (file.startsWith(m.baseDir))
                module = m;
        }
        if (!module)
            return;
        const part = path.relative(module.baseDir, file);
        return { module, part };
    }
    close() {
        this.watcher.close();
    }
}
//# sourceMappingURL=index.js.map