import { FSWatcher } from 'chokidar';
import { startWatch } from '../watcher/index.js';
import { ModuleGraph } from './index.js';
import { getWorkspaceScanner } from './workspaceScanner/index.js';
import { InvalidModuleError } from './module.js';
vi.mock('chokidar', () => ({
    FSWatcher: vi.fn().mockImplementation(() => ({
        on: vi.fn(),
        close: vi.fn(),
    })),
}));
vi.mock('../watcher/index.js', () => ({
    startWatch: vi.fn(),
}));
vi.mock('./workspaceScanner/index.js', () => ({
    getWorkspaceScanner: vi.fn(),
}));
describe('ModuleGraph', () => {
    let config;
    let workspaceScanner;
    let moduleGraph;
    let watcher;
    let modules;
    let logger;
    beforeEach(() => {
        vi.clearAllMocks();
        modules = [
            {
                baseDir: '/tmp/ns/name',
                id: vi.fn().mockReturnValue('ns/name'),
                addOrUpdatePart: vi.fn().mockReturnValue(true),
                deletePart: vi.fn(),
                getModuleBoundary: vi.fn().mockReturnValue('boundary'),
                getEntryFile: vi.fn().mockReturnValue('name.js'),
            },
        ];
        config = { paths: ['src'] };
        logger = {
            error: vi.fn(),
            debug: vi.fn(),
            info: vi.fn(),
        };
        workspaceScanner = {
            getModules: vi.fn().mockReturnValue(modules),
        };
        watcher = new FSWatcher();
        vi.mocked(startWatch).mockReturnValue(watcher);
        vi.mocked(getWorkspaceScanner).mockReturnValue(workspaceScanner);
        moduleGraph = new ModuleGraph(config, logger);
    });
    it('should initialize correctly', () => {
        expect(startWatch).toHaveBeenCalledWith(config.paths);
        expect(getWorkspaceScanner).toHaveBeenCalledWith(config, logger);
        expect(logger.debug).toHaveBeenCalledWith('[Module Graph] Found 1 LWC components in the current workspace.');
    });
    it('should handle file change events through watcher', () => {
        const filePath = '/tmp/ns/name/cmp.js';
        const updateListener = vi.fn();
        moduleGraph.addListener('update', updateListener);
        vi.mocked(watcher).on.mock.calls.find((c) => c[0] === 'change')[1](filePath);
        expect(modules[0].addOrUpdatePart).toHaveBeenCalledWith('cmp.js');
        expect(logger.info).toHaveBeenCalledWith(`[LWC Dev Server] File 'cmp.js' modified - updating.`);
        expect(updateListener).toHaveBeenCalledWith('boundary');
    });
    it('should handle file add events through watcher', () => {
        const filePath = '/tmp/ns/name/cmp1.js';
        const addListener = vi.fn();
        moduleGraph.addListener('update', addListener);
        vi.mocked(watcher).on.mock.calls.find((c) => c[0] === 'add')[1](filePath);
        expect(modules[0].addOrUpdatePart).toHaveBeenCalledWith('cmp1.js');
        expect(logger.info).toHaveBeenCalledWith(`[LWC Dev Server] File 'cmp1.js' added - updating.`);
        expect(addListener).toHaveBeenCalledWith('name.js');
    });
    it('should handle file delete events through watcher', () => {
        const filePath = '/tmp/ns/name/cmp.js';
        const addListener = vi.fn();
        moduleGraph.addListener('update', addListener);
        vi.mocked(watcher).on.mock.calls.find((c) => c[0] === 'unlink')[1](filePath);
        expect(modules[0].deletePart).toHaveBeenCalledWith('cmp.js');
        expect(logger.info).toHaveBeenCalledWith(`[LWC Dev Server] File '/tmp/ns/name/cmp.js' deleted - updating.`);
        expect(addListener).toHaveBeenCalledWith('name.js');
    });
    it('should handle file delete events when entry file is deleted', () => {
        const filePath = '/tmp/ns/name/name.js';
        const deleteModule = vi.fn();
        moduleGraph.addListener('delete-module', deleteModule);
        vi.mocked(modules[0].getEntryFile).mockImplementation(() => {
            throw new InvalidModuleError();
        });
        vi.mocked(watcher).on.mock.calls.find((c) => c[0] === 'unlink')[1](filePath);
        expect(modules[0].deletePart).toHaveBeenCalledWith('name.js');
        expect(logger.info).toHaveBeenCalledWith(`[LWC Dev Server] File '/tmp/ns/name/name.js' deleted - updating.`);
        expect(moduleGraph.modules.has('ns/name')).toBe(false);
        expect(deleteModule).toHaveBeenCalledWith('ns/name');
    });
    it('should identify stale modules', () => {
        const modules = [
            { id: 'ns/name', part: 'cmp.js', hash: 'hash1' },
            { id: 'ns/other', part: 'cmp.js', hash: 'hash2' },
        ];
        const module1 = {
            id: vi.fn().mockReturnValue('ns/name'),
            getPart: vi.fn().mockReturnValue({ moduleSourceHash: 'hash1' }),
            getEntryFile: vi.fn().mockReturnValue('entryFile1'),
        };
        const module2 = {
            id: vi.fn().mockReturnValue('ns/other'),
            getPart: vi.fn().mockReturnValue({ moduleSourceHash: 'differentHash' }),
            getEntryFile: vi.fn().mockReturnValue('entryFile2'),
        };
        moduleGraph.modules.set('ns/name', module1);
        moduleGraph.modules.set('ns/other', module2);
        const staleModules = moduleGraph.getStaleModules(modules);
        expect(logger.debug).toHaveBeenCalledWith(`[Module Graph] Following modules have a different version: \n\t ns/other`);
        expect(staleModules).toEqual(new Set(['entryFile2']));
    });
    it('should close the watcher', () => {
        moduleGraph.close();
        expect(moduleGraph.watcher.close).toHaveBeenCalled();
    });
});
//# sourceMappingURL=index.spec.js.map