import path from 'path';
import * as fs from 'fs/promises';
import { getModuleHash } from '../util/fileUtil.js';
import { isFileInValidLwcFolder } from './workspaceScanner/fileReader.js';
/**
 * Represents a Module entry per LWC component
 */
export class Module {
    constructor(baseDir, namespace, name, 
    /**
     * Various parts of the module.
     * For example:
     * {
     *      "myComponent.js": { moduleHash: "xxx" },
     *      "myComponent.html": { moduleHash: "xxy" },
     *      "util/util.js": { moduleHash: "xxz" },
     * }
     */
    moduleParts) {
        this.baseDir = baseDir;
        this.namespace = namespace;
        this.name = name;
        this.moduleParts = moduleParts;
    }
    id(part) {
        const id = `${this.namespace}/${this.name}`;
        return part ? path.join(id, part) : id;
    }
    /**
     * Returns the entry file of this module. Not cached because it can change overtime.
     * @returns entry file of the module
     */
    getEntryFile() {
        return this.id(this.getEntryPart());
    }
    getEntryPart() {
        const potentialEntryFiles = ['js', 'ts', 'css'].map((ext) => `${this.name}.${ext}`);
        const entryFile = potentialEntryFiles.find((entry) => this.moduleParts[entry]);
        if (!entryFile) {
            throw new InvalidModuleError();
        }
        return entryFile;
    }
    /**
     * Identify which modulePath that represents the boundary of the changed module.
     * In the case of .html files, the module boundary is the template file itself. swapTemplate()
     *  is called and module swap is done.
     * In the case of .css files, the module boundary is the css file itself, swapStyle() is called
     *  and module swap is done.
     * In the case of .js files, the module boundary is the main entry file of the module, all the files
     *  reachable from the entry module are bundled and swapComponent invoked.
     *  Since JS hot swap is currently supporting components, this focused update of the main entry file
     *  is acceptable.
     * @param modulePart The module that changed
     * @return The modulePath of the boundary that needs to be updated.
     */
    getModuleBoundary(modulePart) {
        const { ext } = path.parse(modulePart);
        if (['.html', '.css'].includes(ext)) {
            return this.id(modulePart);
        }
        return this.getEntryFile();
    }
    /**
     * Fetch files and source related to the modulePath that is being hot loaded.
     * @param {string} modulePart The formatted path for the file being compiled.
     * @returns
     */
    async getSources(modulePart) {
        const { ext } = path.parse(modulePart);
        const modulePaths = [];
        // HTML and CSS can be hot-swapped independently, so we compile only those.
        const canCompileIndependently = ['.html', '.css'].includes(ext);
        if (canCompileIndependently) {
            modulePaths.push(modulePart);
        }
        else if (['.js', '.ts'].includes(ext)) {
            // but if JS changes, we recompile the entire module
            modulePaths.push(...Object.keys(this.moduleParts));
        }
        const sources = await Promise.all(modulePaths.map(async (modulePath) => [
            modulePath,
            await fs.readFile(path.join(this.baseDir, modulePath), 'utf-8'),
        ]));
        // In cases where we compile HTML and CSS independently,
        // Rollup needs an entry file matching the component name.
        // We can't pass the actual entry file because it'll consume the above
        // HTML and CSS and export just the component. We need the actuall CSS/HTML to
        // be exported so we can hot-swap.
        // Here, we pass a dummy entry file that just re-exports everything
        // This satisfies rollup and gives us the exported CSS and HTML
        // One caveat though: we don't want HMR transformations to run on this dummy entry file
        // which is why we have this __LWC_HMR_DUMMY_IMPORTER marker
        if (canCompileIndependently) {
            sources.push([
                this.name + '.js',
                `/* __LWC_HMR_DUMMY_IMPORTER */ export * from './${modulePaths[0]}'; import Default from './${modulePaths[0]}'; export default Default;`,
            ]);
        }
        return Object.fromEntries(sources);
    }
    /**
     * Delets the part and returns the entry file of the module
     * @param part part of the module to be deleted
     */
    deletePart(part) {
        delete this.moduleParts[part];
    }
    getPart(part) {
        return this.moduleParts[part];
    }
    /**
     * Updates the part of this module by recomputing its hash
     * @param part part of the module to update
     * @returns true if successful or false if it wasn't needed
     */
    addOrUpdatePart(part) {
        if (!isFileInValidLwcFolder(part))
            return false;
        const newModuleHash = getModuleHash(path.join(this.baseDir, part));
        if (this.moduleParts[part] && this.moduleParts[part].moduleSourceHash === newModuleHash)
            return false;
        this.moduleParts[part] = { moduleSourceHash: newModuleHash };
        return true;
    }
}
export class InvalidModuleError extends Error {
}
//# sourceMappingURL=module.js.map