import * as fs from 'fs/promises';
import { getModuleHash } from '../util/fileUtil';
import { Module, InvalidModuleError } from './module';
vi.mock('fs/promises');
vi.mock('../util/fileUtil');
describe('Module', () => {
    let module;
    const baseDir = '/base/dir';
    const namespace = 'namespace';
    const name = 'name';
    beforeEach(() => {
        const moduleParts = {
            'name.js': { moduleSourceHash: 'hash1' },
            'name.html': { moduleSourceHash: 'hash2' },
            'util/util.js': { moduleSourceHash: 'hash3' },
        };
        module = new Module(baseDir, namespace, name, moduleParts);
        vi.resetAllMocks();
    });
    it('should return the correct id', () => {
        expect(module.id()).toBe('namespace/name');
        expect(module.id('part')).toBe('namespace/name/part');
    });
    it('should return the entry file', () => {
        expect(module.getEntryFile()).toBe('namespace/name/name.js');
    });
    it('should throw InvalidModuleError if no entry file is found', () => {
        const emptyModule = new Module(baseDir, namespace, name, {});
        expect(() => emptyModule.getEntryFile()).toThrow(InvalidModuleError);
    });
    it('should return the correct module boundary for .html and .css files', () => {
        expect(module.getModuleBoundary('name.html')).toBe('namespace/name/name.html');
        expect(module.getModuleBoundary('name.css')).toBe('namespace/name/name.css');
    });
    it('should return the entry file as module boundary for .js and .ts files', () => {
        expect(module.getModuleBoundary('name.js')).toBe('namespace/name/name.js');
        expect(module.getModuleBoundary('name.ts')).toBe('namespace/name/name.js');
    });
    it('should fetch sources correctly', async () => {
        vi.mocked(fs.readFile).mockResolvedValue('file content');
        const sources = await module.getSources('name.js');
        expect(sources).toEqual({
            'name.js': 'file content',
            'name.html': 'file content',
            'util/util.js': 'file content',
        });
    });
    it('should fetch individual sources correctly', async () => {
        vi.mocked(fs.readFile).mockResolvedValue('file content');
        const sources = await module.getSources('name.html');
        expect(sources).toEqual({
            'name.js': `/* __LWC_HMR_DUMMY_IMPORTER */ export * from './name.html'; import Default from './name.html'; export default Default;`,
            'name.html': 'file content',
        });
    });
    it('should delete a part', () => {
        module.deletePart('name.html');
        expect(module.getPart('name.html')).toBeUndefined();
    });
    it('should add or update a part', () => {
        vi.mocked(getModuleHash).mockReturnValue('newHash');
        expect(module.addOrUpdatePart('name.js')).toBe(true);
        expect(module.getPart('name.js')).toEqual({ moduleSourceHash: 'newHash' });
    });
    it('should not update a part if the hash is the same', () => {
        vi.mocked(getModuleHash).mockReturnValue('hash1');
        expect(module.addOrUpdatePart('name.js')).toBe(false);
    });
});
//# sourceMappingURL=module.spec.js.map