import { existsSync, readdirSync } from 'node:fs';
import path from 'node:path';
import { getModuleHash } from '../../util/fileUtil.js';
/**
 * Is the file a valid LWC component part?
 * @param {string} file
 * @param {string} componentName Component name
 * @returns {boolean}
 */
function isValidLwcFile(file) {
    if (['.ts', '.js', '.html', '.css'].includes(path.extname(file)))
        return true;
    return false;
}
/**
 * Files in folders with __ or __docs__ aren't supposed to be included in LWCs sources
 * @param file file path
 * @returns true if file is in a valid LWC folder
 */
export function isFileInValidLwcFolder(file) {
    return !path
        .dirname(file)
        .split(path.sep)
        .some((dir) => dir === '__' || dir === '__docs__');
}
/**
 * Should the directory be processed as a valid LWC module part?
 */
function shouldProcessDirectory(file) {
    const name = path.basename(file);
    return !name.startsWith('__') || name === '__docs__';
}
/**
 * Get valid LWC files recursively from a directory
 * @param {string} dir Component Directory
 * @returns {Promise<string[]>} valid LWC file paths
 */
function getFiles(dir) {
    if (!shouldProcessDirectory(dir)) {
        return [];
    }
    const dirents = readdirSync(dir, { withFileTypes: true });
    return dirents.flatMap((dirent) => {
        const file = path.resolve(dir, dirent.name);
        if (dirent.isDirectory()) {
            return getFiles(file);
        }
        return isValidLwcFile(file) ? [file] : [];
    });
}
/**
 * Verify if a given directory represents an LWC component directory by checking if a valid entry file exists.
 * @param entryPoint The directory to test.
 * @param componentName The component name.
 * @returns {boolean} true if directory has an entry file.
 */
function isValidEntryPoint(entryPoint, componentName) {
    const jsEntryFile = path.join(entryPoint, componentName + '.js');
    const tsEntryFile = path.join(entryPoint, componentName + '.ts');
    const cssOnlyModuleEntryFile = path.join(entryPoint, componentName + '.css');
    return existsSync(jsEntryFile) || existsSync(tsEntryFile) || existsSync(cssOnlyModuleEntryFile);
}
/**
 * Get all parts of an LWC component.
 * @param componentDir A directory representing an LWC Component.
 * @param namespace String representing the namespace of the component
 * @param ownerModuleId Owner module in format like <namespace>/<component-name>.
 * @returns A map representing all valid parts of the LWC with the short path as the key and full
 *  canonical path as value.
 */
export function getComponentParts(componentDir) {
    const name = path.basename(componentDir);
    if (!isValidEntryPoint(componentDir, name))
        return undefined;
    const files = getFiles(componentDir);
    return Object.fromEntries(files.map((file) => {
        return [
            `${path.relative(componentDir, file)}`,
            { moduleSourceHash: getModuleHash(file) },
        ];
    }));
}
//# sourceMappingURL=fileReader.js.map