import type { Lifecycle } from '@salesforce/core';
export declare const MAX_CLIENT_COUNT = 5;
/**
 * Define the type of workspace.
 * Internal is for salesforce internal developers.
 * SfCli is for the Salesforce CLI project setup.
 */
export declare enum Workspace {
    Internal = "internal",
    SfCli = "SalesforceCLI",
    Mrt = "mrt"
}
/**
 * Config for HTTPS server
 * Technically all the supported fields that `require('ndoe:https').createServer` accepts can be passed here,
 * https://nodejs.org/api/https.html#httpscreateserveroptions-requestlistener
 *
 * but we handpicked the ones that are most used here.
 * Usually users either provide a cert and a key or pfx and a passphrase
 */
export type HttpsConfig = {
    cert?: string;
    key?: string;
    pfx?: Buffer;
    passphrase?: string;
    port: number;
};
/**
 * Interface representing the LWC Local Development server config.
 */
export interface ServerConfig {
    /**
     * The root directory for the project, config files and path will be referenced as relative to this directory
     * For internal developers: core home
     * For Salesforce CLI customers: The Salesforce DX Project dir
     */
    rootDir: string;
    /** Port which the HTTP Server will bind to. */
    port: number;
    /**
     * Component paths to watch
     * For SF CLI: force-app/main/default
     */
    paths: string[];
    workspace: Workspace;
    https?: HttpsConfig;
    /**
     * maximum number of clients that can simultaneously connect to the dev server.
     * This avoids the dev server from being overwhelmed(DoS) by multiple hmr-client connecting at the same time.
     * default value is {@link MAX_CLIENT_COUNT}
     */
    maxClientCount?: number;
    /**
     * Identity token for the server to auth
     */
    identityToken?: string;
    /**
     * Paths to exclude from watching
     * For Internal Workspace: should be the path to the directory of the component source
     * TODO: Add support for Salesforce CLI
     */
    excludePaths?: string[];
    /**
     * Asynchronous event listener and emitter.
     * Primarily used for gathering telemetry when used in the context of sf cli environment.
     */
    lifecycle?: Lifecycle;
    /**
     * Type of client connecting to the dev server. Eg: ios, android, desktop, etc.
     * This is used for telemetry purposes and does not affect the functionality of the dev server.
     */
    clientType?: string;
    /**
     * namespace to be used for Salesforce CLI instead of the default 'c'
     */
    namespace?: string;
}
